/*CoolDima compute cooling due to level 2 lines */
#include "cddefines.h"
#include "taulines.h"
#include "phycon.h"
#include "ionfracs.h"
#include "colstrgbar.h"
#include "converge.h"
#include "level2.h"
#include "putcs.h"

void CoolDima(void)
{
	long int i, 
	  ion,
	  nelem;
	double cs;
	static float TeEvalCS = -1.f;

#	ifdef DEBUG_FUN
	fputs( "<+>CoolDima()\n", debug_fp );
#	endif

	/* check whether we need to reevaluate the collision strenghts.
	 * Do so if large change in temperature  or any stage of ionization has been lowered. */
	if( (conv.lgSearch || conv.lgIonStageTrimed || 
		fabs(phycon.te-TeEvalCS)/phycon.te > 0.05) )
	{
		for( i=0; i < nWindLine; i++ )
		{
			/* only evaluate cs if positive abundance */
			ion = (long)(TauLine2[i][IPLNIONSTG-1] + 0.00001);
			nelem = (long)(TauLine2[i][IPLNNELEM-1] + 0.00001);

			if( xIonFracs[ion][nelem-1] > 0. )
			{
				/* now generate the collision strength */
				cs = ColStrGBar(&TauLine2[i][0]);
			}
			else
			{
				cs = 1.;
			}
			/* now put the cs into the line array */
			PutCS(cs,&TauLine2[i][0]);
		}
		TeEvalCS = phycon.te;
	}

	/* now always update the cooling since this adds ots flux */
	for( i=0; i < nWindLine; i++ )
	{
		/* we need to call this even if nothing present since then sets zero
		 * ignore all lines with negative atomic number
		 * >>chng 97 aug 30, get rid of test for non-pos ipLnNelem
		 * pointer tells level2 to use existing WineEtc labels */
		level2(&TauLine2[i][0]);
	}

#	ifdef DEBUG_FUN
	fputs( " <->CoolDima()\n", debug_fp );
#	endif
	return;
}

