/*CoolPhos compute phosphorus cooling */
#include "cddefines.h"
#include "taulines.h"
#include "tepowers.h"
#include "level2.h"
#include "putcs.h"
#include "level3.h"
#include "coolmetals.h"

void CoolPhos()
{
	double cs;

#	ifdef DEBUG_FUN
	fputs( "<+>CoolPhos()\n", debug_fp );
#	endif

	/* [P II] 60.64, 32.87 microns
	 * cs, As from
	 * >>refer Mendoza, C., & Zeippen, C.J., 1982, MNRAS 199, 1025
	 * >>refer Krueger, T.K., and Czyzak, S.J., 1970,
	 * >>refer Proc Roy Soc London A 318, 531 */
	PutCS(1.587,&TauLines[ipP0260][0]);
	PutCS(3.566,&TauLines[ipP0233][0]);
	PutCS(1.0,TauDummy);

	/* level3(  t10,t21,t20) */
	level3(&TauLines[ipP0260][0],&TauLines[ipP0233][0],TauDummy);

	/* [P III] 17.885 microns
	 * CS, A from
	 * >>refer Kaufman, V., & Sugar, J., 1986, J Phys Chem Ref Data 15, 321
	 * >>refer Krueger, T.K., and Czyzak, S.J., 1970,
	 * >>refer Proc Roy Soc London A 318, 531 */
	PutCS(1.859,&TauLines[ipP0318][0]);
	level2(&TauLines[ipP0318][0]);

	/* [P VII] 1.374 microns
	 * cs from 
	 * >>refer Saraph, H.E. & Tully, J.A. 1994, A&AS, 107, 29 */
	PutCS(0.27,&TauLines[ipP713][0]);

	level2(&TauLines[ipP713][0]);

	/* [P VIII] 4.85, 1.735 microns */
	PutCS(0.30,&TauLines[ipP848][0]);
	PutCS(0.97,&TauLines[ipP817][0]);
	PutCS(0.26,TauDummy);

	/* j levels inverted */
	level3(&TauLines[ipP817][0],&TauLines[ipP848][0],TauDummy);

	/* [P X] 2.708, 1.868 microns */
	cs = MIN2(0.33,0.892/(TePowers.te10/TePowers.te001));
	PutCS(cs,&TauLines[ipP1027][0]);

	cs = MIN2(1.082,5.949/(TePowers.te20/TePowers.te03*TePowers.te003));
	PutCS(cs,&TauLines[ipP1018][0]);

	cs = MIN2(0.33,3.054/(TePowers.te20*TePowers.te02*TePowers.te005));
	PutCS(cs,TauDummy);

	level3(&TauLines[ipP1027][0],&TauLines[ipP1018][0],TauDummy);


#	ifdef DEBUG_FUN
	fputs( " <->CoolPhos()\n", debug_fp );
#	endif
	return;
}

