/*escpcn continuum escape probability */
#include "cddefines.h"
#include "qg32.h"
#include "conrec.h"
#include "conesc.h"
#include "chnukt.h"
#include "escpcn.h"

double escpcn(double tau, 
  double hnukt)
{
	double dinc, 
	  escpcn_v, 
	  sumesc, 
	  sumrec;

#	ifdef DEBUG_FUN
	fputs( "<+>escpcn()\n", debug_fp );
#	endif

	/* computes continuum escape probabilities */
	if( tau < 0.01 )
	{
		escpcn_v = 1.;
		
#	ifdef DEBUG_FUN
		fputs( " <->escpcn()\n", debug_fp );
#	endif
		return( escpcn_v );
	}
	else if( hnukt > 1. && tau > 100. )
	{
		escpcn_v = 1e-20;
		
#	ifdef DEBUG_FUN
		fputs( " <->escpcn()\n", debug_fp );
#	endif
		return( escpcn_v );
	}

	chnukt.ContTkt = (float)hnukt;
	chnukt.ctau = (float)tau;

	dinc = 10./hnukt;
	sumrec = qg32(1.,1.+dinc,conrec);
	sumesc = qg32(1.,1.+dinc,conesc);

	if( sumrec > 0. )
	{
		escpcn_v = sumesc/sumrec;
	}
	else
	{
		escpcn_v = 0.;
	}


#	ifdef DEBUG_FUN
	fputs( " <->escpcn()\n", debug_fp );
#	endif
	return( escpcn_v );
}

