/*FirstDR derive thickness of first zone */
#include <float.h>
#include <string.h>
#include "cddefines.h"
#include "cdminmax.h"/* vfmin */
#define	Z	1.0001
#include "itercnt.h"
#include "wind.h"
#include "sdrmin.h"
#include "stopcalc.h"
#include "trace.h"
#include "drmnon.h"
#include "rfield.h"
#include "drmin.h"
#include "fluct.h"
#include "filfac.h"
#include "phycon.h"
#include "radius.h"
#include "opac.h"
#include "ipoint.h"
#include "h.h"
#include "didz.h"
#include "densty.h"
#include "pressure.h"
#include "tabden.h"
#include "firstdr.h"

void FirstDR(void)
{
	long int i ,
		ip;

	double accel, 
	  BigOpacity, 
	  dr912, 
	  drOpacity ,
	  drTabDen, 
	  dradf, 
	  drcol, 
	  drthrm, 
	  factor, 
	  winddr;

#	ifdef DEBUG_FUN
	fputs( "<+>FirstDR()\n", debug_fp );
#	endif

	/***********************************************************************
	 *
	 * wind model, use acceleration length                          
	 *
	 ***********************************************************************/

	if( wind.windv > 0. )
	{
		/* evaluate total pressure, although value not used (so stuffed into dr912) */
		dr912 = TotalPressure();
		accel = 1.3e-10*densty.pden*densty.wmole;
		winddr = POW2(wind.windv)/25./accel;
	}
	else
	{
		winddr = 1e30;
	}

	/* key off of Lyman continuum optical depth */
	if( StopCalc.taunu > 0.99 && StopCalc.taunu < 3. )
	{
		dr912 = StopCalc.tauend/6.3e-18/(h.hi*filfac.FillFac)*Z/50.;
	}
	else
	{
		dr912 = 1e30;
	}

	/***********************************************************************
	 *
	 * key off of column density; total, neutral, or ionized                          
	 *
	 ***********************************************************************/

	if( StopCalc.HColStop < 5e29 )
	{
		/* this is useful for very thin columns, normally larger than 1st DR */
		drcol = log10(StopCalc.HColStop) - log10(phycon.hden*filfac.FillFac* 20.);
	}
	else if( StopCalc.colpls < 5e29 )
	{
		/* ionized column density */
		drcol = log10(StopCalc.colpls) - log10(h.hii*filfac.FillFac* 20.);
	}
	else if( StopCalc.colnut < 5e29 )
	{
		/* neutral column denisty */
		drcol = log10(StopCalc.colnut) - log10(h.hi*filfac.FillFac*50.);
	}
	else
	{
		/* not used */
		drcol = 30.;
	}
	/* finally convert the drived column density to linear scale */
	drcol = pow(10.,MIN2(35.,drcol));

	/***********************************************************************
	 *
	 * key off of density or abundance fluctuations, must be small part of wavelength                          
	 *
	 ***********************************************************************/

	if( fluct.flong != 0. )
	{
		/* flong set => density fluctuations */
		dradf = 6.283/fluct.flong/10.;
		dradf = vfmin(dradf,radius.router[IterCnt.iter-1]*
		  Z,drcol,dr912,FEND);
	}
	else
	{
		dradf = FLT_MAX;
	}

	/***********************************************************************
	 *
	 * find largest opacity, to keep the first zone optical depth 1
	 * this is usually the physics that sets the first zone thickness
	 *
	 ***********************************************************************/

	/* >>>chng 99 jun 25, this is to simulate behavior of code before extension
	 * of continuum arrray to 1e-8 ryd */
	ip = ipoint(1e-5);

	/* find largest opacity */
	BigOpacity = 0.;
	for( i=ip; i < rfield.nflux; i++ )
	{
		BigOpacity = MAX2(BigOpacity,opac.opac[i]);
	}

	/* this cannot possibly happen */
	assert( BigOpacity > 0. );

	/* drChange set with set didz command, is only number set with this command,
	 * default in zerologic is 0.15 
	 * set drad to small part of*/
	drOpacity = (didz.drChange/100.)/BigOpacity/filfac.FillFac;

	/* initialize something in following routine? vars not used here */
	/* ContRate(&freqm,&opacm); 98 jan 24, kill, see what happens */

	/***********************************************************************
	 *
	 * thermalization length of typical lines
	 *
	 ***********************************************************************/

	drthrm = 1.5e31/MAX2(1.,phycon.hden*phycon.hden);

	/***********************************************************************
	 *
	 * make sure we resolve initial structure in tabden command
	 * if interpolated table we need to make sure that we resolve the 
	 * initial changes in the structure
	 *
	 ***********************************************************************/

	if( strcmp(pressure.chCPres,"DLW2") == 0 )
	{
		drTabDen = 1.;
		i = 1;
		factor = 0.;
		while( i < 100 && factor < 0.05 )
		{
			/* check densities at ever larger dr's, until factor becomes more than 5% */
			factor = phycon.hden/
				tabden(radius.Radius+drTabDen, drTabDen );
			/* density change can be positive or negative sign */
			factor = fabs(factor-1.);
			drTabDen *= 2.;
			i += 1;
		}
		drTabDen /= 2.;
	}
	else
	{
		drTabDen = 1e30;
	}

	/***********************************************************************
	 *
	 * we have now generated range of estimates of first thickness,
	 * now choose smallest of the group
	 *
	 ***********************************************************************/

	/* radius div by 20, to prevent big change in iron ionization for high ioniz gas,
	 * this is also the ONLY place that sphericity comes in */
	radius.drad = vfmin( drOpacity  , radius.Radius/20.,
		radius.router[IterCnt.iter-1]/10.   ,   drcol,dr912,
		sdrminCom.sdrmax  ,  drthrm  ,  winddr  ,  dradf  ,  drTabDen  ,  FEND);

	/* option to set lower limit to zone thickness, with set drmin command*/
	radius.drad = MAX2((float)(radius.drad),sdrminCom.sdrmin);

	/* drMinimum is smallest acceptable DRAD, and is 1/100 OF DRAD(1) */
	/* this can be turned off by GLOB command */
	if( drmnon.lgDrMnOn )
	{
		drmin.drMinimum = (float)(radius.drad/1e7);
	}
	else
	{
		drmin.drMinimum = 0.;
	}

	if( sdrminCom.lgSMinON )
	{
		drmin.drMinimum = sdrminCom.sdrmin;
	}

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, 
			" FirstDR called, finds dr=%13.5e drMinimum=%12.3e sdrmin=%10.2e sdrmax=%10.2e\n", 
		  radius.drad, drmin.drMinimum, sdrminCom.sdrmin, sdrminCom.sdrmax );
	}


#	ifdef DEBUG_FUN
	fputs( " <->FirstDR()\n", debug_fp );
#	endif
	return;
}

