/*HydroOTS evaluate model hydrogen atom ots rates */
#include "cddefines.h"
#include "taulines.h"
#include "hydrogenic.h"
#include "ionrange.h"
#include "pbowen.h"
#include "elmton.h"
#include "phycon.h"
#include "ionfracs.h"
#include "nh.h"
#include "rfield.h"
#include "heots.h"
#include "addotscon.h"
#include "addotslin.h"
/* the Bowen HeII yield */
#define BOWEN 0.3f

void HydroOTS(void)
{
	long int ipHi, 
	  ipLo, 
	  ipZ, 
	  n;
	float bwnfac ,
	  ots660 ,
	  otsnew;
	

#	ifdef DEBUG_FUN
	fputs( "<+>HydroOTS()\n", debug_fp );
#	endif

	/**************************************************************************
	 *
	 * the bowen HeII - OIII fluorescense problem
	 *
	 **************************************************************************/

	ipZ = 1;
	if( elmton.lgElmtOn[ipZ] )
	{
		/* conversion per unit atom to OIII, at end of sub we divide by it,
		 * to fix lines back to proper esc/dest probs */
		bwnfac = BOWEN * (1.f - HydroLines[ipZ][IP2P][IP1S][IPLNESCP-1] );

		/* escaping part that will be added to the outward beam in metdif,
		 * this uses ipLnEsc instead of ipLnDesp */
		heots.esc660 = HydroLines[ipZ][IP2P][IP1S][IPLNAUL-1]*
		  HydroLines[ipZ][IP2P][IP1S][IPLNPOPU-1]*xIonFracs[ipZ+2][ipZ]*
		  HydroLines[ipZ][IP2P][IP1S][IPLNESCP-1] *BOWEN*2.0f * 0.8f ;

		/* the last factor accounts for fact that two photons are produced,
		 * and the branching ratio */
		ots660 = HydroLines[ipZ][IP2P][IP1S][IPLNAUL-1]*
		  HydroLines[ipZ][IP2P][IP1S][IPLNPOPU-1]*xIonFracs[ipZ+2][ipZ]*
		  HydroLines[ipZ][IP2P][IP1S][IPLNDESP-1] *BOWEN*2.0f * 0.8f ;

		/* now add this to the ots field */
		AddOTSLin(ots660 , pbowen.ip660 );

		/* decrease the destruction prob by the amount we will add elsewhere,
		 * ok since dest probs updated on every iteration*/
		HydroLines[ipZ][IP2P][IP1S][IPLNDESP-1] *= bwnfac;
	}

	else
	{
		bwnfac = 1.;
	}

	/* make ots fields due to hydrogen lines and continua */

	/* loop over all elements */
	for( ipZ=0; ipZ < LIMELM; ipZ++ )
	{
		/* do if this stage exists */
		if( (IonRange.IonHigh[ipZ] == ipZ + 2)  )
		{

			/**************************************************************************
			 *
			 * generate hydrogenic line ots rates 
			 *
			 **************************************************************************/

			/* now loop over all possible levels, but cannot include two photon
			 * since there is no pointer to this continuum */
			for( ipHi=IP2P; ipHi <= nhlevel; ipHi++ )
			{
				for( ipLo=IP1S; ipLo < ipHi; ipLo++ )
				{
					/* ots rates, the destp prob was set in hydropesc */
					HydroLines[ipZ][ipHi][ipLo][IPLNOTS-1] = 
						HydroLines[ipZ][ipHi][ipLo][IPLNAUL-1]*
						HydroLines[ipZ][ipHi][ipLo][IPLNPOPU-1]*xIonFracs[ipZ+2][ipZ]*
						HydroLines[ipZ][ipHi][ipLo][IPLNDESP-1];

					assert( HydroLines[ipZ][ipHi][ipLo][IPLNOTS-1] >= 0. );

					/* finally dump the ots rate into the stack */
					AddOTSLin(HydroLines[ipZ][ipHi][ipLo][IPLNOTS-1],
						(long)(HydroLines[ipZ][ipHi][ipLo][IPLNIPCONT-1]) );
				}
			}
			{
				/* debugging code for Lya problems */
				enum {DEBUG = FALSE};
				if( DEBUG )
				{
					if( ipZ==0 ) 
					{
#						include "radius.h"
#						include "opac.h"
						fprintf(ioQQQ,"hlyaots\t%g\t%g\t%g\t%g\n",
							radius.depth ,
							HydroLines[0][2][0][IPLNOTS-1] , 
							opac.opac[(long)(HydroLines[0][2][0][IPLNIPCONT-1])-1] ,
							rfield.otslinNew[(long)(HydroLines[0][2][0][IPLNIPCONT-1])-1] );
					}
				}
			}

			/**************************************************************************
			 *
			 * ots recombination bound-free b-f continua continuum
			 *
			 **************************************************************************/

			/* put in OTS continuum */
			for( n=IP1S; n <= nhlevel; n++ )
			{
				otsnew = hydro.hrec[ipZ][n][ipRecRad]*
					(1.f - hydro.hrec[ipZ][n][ipRecEsc])*phycon.eden*
					xIonFracs[ipZ+2][ipZ];
				assert( otsnew >= 0. );

				/*otsnew = (rfield.otscon[nh.ipHn[ipZ][n]-1]*
					(1.f - damp) + 
					damp*hydro.hrec[ipZ][n][ipRecRad]*
					(1.f - hydro.hrec[ipZ][n][ipRecEsc])*phycon.eden*
					xIonFracs[ipZ+2][ipZ]/
					opac.opac[nh.ipHn[ipZ][n]-1])/2.f;*/

				/* pointer used in this routine is decremented by one there */
				AddOTSCon(otsnew,nh.ipHn[ipZ][n]);
			}
		}
	}

	ipZ = 1;
	if( elmton.lgElmtOn[ipZ] && bwnfac > 0. )
	{
		/* increase the destruction prob by the amount we wdecreased it above */
		HydroLines[ipZ][IP2P][IP1S][IPLNDESP-1] /= bwnfac;
	}


#	ifdef DEBUG_FUN
	fputs( " <->HydroOTS()\n", debug_fp );
#	endif
	return;
}

