/*MakeStatRT do line radiative transfer for static geometry,
 * evaluates escape and destruction probability, 
 * called by HydroPEsc, MakeRT  */
#include <string.h>
#include "cddefines.h"
#include "twoway.h"
#include "showme.h"
#include "rfield.h"
#include "doppvel.h"
#include "fluors.h"
#include "taumin.h"
#include "escinc.h"
#include "dumpline.h"
#include "esa0k2.h"
#include "touton.h"
#include "escgrd.h"
#include "eovrlp.h"
#include "escsub.h"
#include "taulines.h"
#include "converge.h"
#include "makert.h"

void MakeStatRT(float t[NTA] , int lgDoEsc )
{
	int lgGoodTau;
	long int nelem;
	float damp, 
	  velocity;
	char chRedis[5];

#	ifdef DEBUG_FUN
	fputs( "<+>MakeStatRT()\n", debug_fp );
#	endif

	/* this is the routine that computes much of the radiative transfer
	 * physics for lines for static case.  */

	/* iplntauin is 1, so -1 is 0 */
	if( t[IPLNTAUIN-1] < -30. )
	{
		fprintf( ioQQQ, " MakeStatRT called with large negative optical depth.\n" );
		DumpLine(t);
		ShowMe();
		CloseUnits();
		puts( "[Stop in makestatrt]" );
		exit(1);
	}
	/*end sanity check */

	/* this checks if we have overrun the optical depth scale
	 * possible for line to mase, reason for sec test
	 * >>chng 97 jan 8, on iterations where the first iteration had NO species,
	 * but it does exist on later iterations, added test for existence of ion */
	if( (t[IPLNTAUTOT-1]*0.9 - t[IPLNTAUIN-1] < 0. && t[IPLNTAUIN-1] > 0.) && 
		  t[IPLNTAUTOT-1] != tauminCom.taumin )
	{
		/* do not do anything if we have overrun the scale, */
		lgGoodTau = FALSE;
	}
	else
	{
		lgGoodTau = TRUE;
	}

	/* say tau ok if this is first iteration and outward optical depths turned off */
	if( !touton.lgTauOutOn )
	{
		lgGoodTau = TRUE;
	}

	/* atomic number of this element */
	nelem = (long)t[IPLNNELEM-1];

	/* thermal plus turbulent velocity for this element */
	velocity = DoppVel.doppler[nelem-1];

	/* damping constant for the line, save it */
	damp = t[IPLNDAMPREL-1]/velocity;
	t[IPLNDAMP-1] = damp;

	/* static solution - which redistribution function? */
	if( t[IPLNREDIS-1] > 0. )
	{
		/* incomplete redistribution  */
		if( lgDoEsc )
		{
			if( lgGoodTau )
			{
				t[IPLNESCP-1] = (float)(escgrd(t[IPLNTAUIN-1],t[IPLNTAUTOT-1],
				  damp));

				/* inward escaping fraction */
				t[IPLNINWD-1] = twoway.fracin;
			}

			/* update pumping even if over run optical depth scale */
			if( fluors.lgFluor )
			{
				/* continuum pumping rate, A gu/gl abs prob occnum
				 * the "no induced" command causes conpmp to set 0 */
				twoway.wayin = (float)escinc(t[IPLNTAUCON-1],damp);
				t[IPLNPUMP-1] = t[IPLNAUL-1]*t[IPLNGU-1]/t[IPLNGL-1]*twoway.wayin*
				  rfield.occnum[(long)(t[IPLNIPCONT-1])-1];
			}
			else
			{
				t[IPLNPUMP-1] = 0.;
			}
		}

		strcpy( chRedis , "INCO" );
	}

	else if( t[IPLNREDIS-1] < 0. )
	{

		/* complete redistribution - t(ipLnRedis) is negative */
		if( lgDoEsc )
		{
			if( lgGoodTau )
			{
				t[IPLNESCP-1] = (float)escsub(t[IPLNTAUIN-1],t[IPLNTAUTOT-1],
				  damp);

				/* inward escaping fraction */
				t[IPLNINWD-1] = twoway.fracin;
			}

			/* continuum pumping rate
			 * the "no fluor" command causes conpmp to set 0 */
			if( fluors.lgFluor )
			{
				/* wayin = esccom( t(ipLnTauCon),damp )
				 * this is to make it more like C90 was
				 * complete redistribution with doppler core only */
				twoway.wayin = (float)esa0k2(t[IPLNTAUCON-1]);
				t[IPLNPUMP-1] = t[IPLNAUL-1]*t[IPLNGU-1]/t[IPLNGL-1]*twoway.wayin*
				  rfield.occnum[(long)(t[IPLNIPCONT-1])-1];
			}
			else
			{
				t[IPLNPUMP-1] = 0.;
			}
		}

		strcpy( chRedis , "  K2" );
#		if 0
		/* >>>chng 99 apr 07, make label and have only one call to eovrlp */
		/* >>>chng 99 apr 07, take average of old and new if not in search mode */
		t[IPLNDESP-1] = (float)(
			eovrlp( t[IPLNPOPOPC-1],t[IPLNOPAC-1],(long)(t[IPLNIPCONT-1]),
		  velocity,t[IPLNESCP-1],"  K2"));
#		endif
	}
	else
	{
		fprintf( ioQQQ, " MakeStatRt called with redistribution function zero\n" );
		ShowMe();
		CloseUnits();
		puts( "[Stop in makestatrt]" );
		exit(1);
	}

	t[IPLNDESP-1] = (float)(eovrlp(
		/* the abundance of the species */
		t[IPLNPOPOPC-1],
		/* line center opacity in funny units (needs a vel) */
		t[IPLNOPAC-1],
		/* pointer to line in continuum array */
		(long)(t[IPLNIPCONT-1]),
		/* line width in velocity units */
		velocity,
		/* escape probability */
		t[IPLNESCP-1],
		/* redistribution function */
		chRedis));

#	ifdef DEBUG_FUN
	fputs( " <->MakeStatRT()\n", debug_fp );
#	endif
	return;
}
