/*MakeWindRT do line radiative transfer for wind geometry */
#include "cddefines.h"
#include "rfield.h"
#include "doppvel.h"
#include "fluors.h"
#include "esccom.h"
#include "eovrlp.h"
#include "taulines.h"
#include "makert.h"

void MakeWindRT(float t[] , int lgDoEsc)
{
	long int nelem;
	float damp, 
	  velocity;

#	ifdef DEBUG_FUN
	fputs( "<+>MakeWindRT()\n", debug_fp );
#	endif

	/* 
	 * this is the routine that computes much of the radiative transfer
	 * physics for lines in wind case.  Lines diveded into two types,
	 * high quality (ipLnCS1=0) and lower quality g-bar lines
	 */

	/*begin sanity check, confirm pops and cs ok */
	if( t[IPLNCOLOVTOT-1] > 1. || t[IPLNCOLOVTOT-1] < 0. )
	{
		fprintf( ioQQQ, " MakeWindRT insane ipLnColovTOT=%10.2e\n", 
		  t[IPLNCOLOVTOT-1] );
	}
	/*end sanity check */

	/* atomic number of species */
	nelem = (long)t[IPLNNELEM-1];

	/*begin sanity check */
	assert( nelem >= 1 );

	/* line width, thermal + turbulence */
	velocity = DoppVel.doppler[nelem-1];

	/* evaluate and store damping constant */
	damp = t[IPLNDAMPREL-1]/velocity;
	t[IPLNDAMP-1] = damp;

	/* use only one-sided escape probabilities
	 * get escape probability, this gets fractions too */
	if( lgDoEsc )
	{
		t[IPLNESCP-1] = (float)esccom(t[IPLNTAUIN-1],damp);

		/* inward fraction */
		t[IPLNINWD-1] = 0.5;

		/* continuum pumping */
		if( fluors.lgFluor )
		{
			t[IPLNPUMP-1] = t[IPLNAUL-1]*t[IPLNGU-1]/t[IPLNGL-1]*t[IPLNESCP-1]*
			  rfield.occnum[(long)(t[IPLNIPCONT-1])-1];
		}
		else
		{
			t[IPLNPUMP-1] = 0.;
		}
	}

	/* get destruction probability */
	t[IPLNDESP-1] = (float)(eovrlp(t[IPLNPOPOPC-1],t[IPLNOPAC-1],(long)(t[IPLNIPCONT-1]),
	  velocity,t[IPLNESCP-1], "  K2"));

#	ifdef DEBUG_FUN
	fputs( " <->MakeWindRT()\n", debug_fp );
#	endif
	return;
}

