/*ParseRadius parse the radius command */
#include <string.h>
#include "cddefines.h"
#define	PARSCL	18.489396
#include "varypar.h"
#include "radius.h"
#include "input.h"
#include "ffmtread.h"
#include "lgmatch.h"
#include "parse.h"

void ParseRadius(char *chCard, 
  float *ar1)
{
	int lgEOL, 
	  lgR2set, 
	  lgRLog;
	long int i;
	double a, 
	  convl;

#	ifdef DEBUG_FUN
	fputs( "<+>ParseRadius()\n", debug_fp );
#	endif

	/* log of inner and outer radii, default second=infinity,
	 * if R2<R1 then R2=R1+R2
	 * there is an optional keyword, "PARSEC" on the line, to use PC as units */
	if( lgMatch("PARS",chCard) )
	{
		convl = PARSCL;
	}
	else
	{
		convl = 0.;
	}

	/* if linear appears on line, then radius is linear, otherwise, log */
	if( lgMatch("LINE",chCard) )
	{
		lgRLog = FALSE;
	}
	else
	{
		lgRLog = TRUE;
	}

	i = 5;
	*ar1 = (float)FFmtRead(chCard,&i,76,&lgEOL);
	if( lgEOL )
	{
		fprintf( ioQQQ, " There should have been a number on this line.  Sorry.\n" );
		CloseUnits();
		puts( "[Stop in ParseRadius]" );
		exit(1);
	}

	/* option for linear or log radius */
	if( lgRLog )
	{
		*ar1 += (float)convl;
	}
	else
	{
		*ar1 = (float)(log10(*ar1) + convl);
	}

	radius.Radius = pow(10.,*ar1);

	/* check for second number, which indicates thickness of model */
	a = (float)FFmtRead(chCard,&i,76,&lgEOL);
	if( lgEOL )
	{
		/* not set */
		lgR2set = FALSE;
	}
	else
	{
		/* outer radius is set, */
		lgR2set = TRUE;

		/* log or linear option is still in place */
		if( lgRLog )
		{
			a += convl;
		}

		else
		{
			a = log10(a) + convl;
		}

		a = pow(10.,a);
		/* check whether it was thickness or outer radius */
		if( a > radius.Radius )
		{
			radius.router[0] = a - radius.Radius;
		}

		else
		{
			radius.router[0] = a;
		}

		for( i=1; i < ITRDIM; i++ )
		{
			radius.router[i] = radius.router[0];
		}
	}

	/* vary option */
	if( VaryPar.lgVarOn )
	{
		/* pointer to where to write */
		VaryPar.nvfpnt[VaryPar.nparm] = input.nRead;
		VaryPar.vincr[VaryPar.nparm] = 0.5;

		if( lgR2set )
		{
			strcpy( chVaryPar.chVarFmt[VaryPar.nparm], "RADIUS %f depth or outer R %f" );
			VaryPar.nvarxt[VaryPar.nparm] = 2;
			/* second number is thickness or outer radius */
			VaryPar.vparm[1][VaryPar.nparm] = (float)log10(radius.router[0]);
		}
		else
		{
			strcpy( chVaryPar.chVarFmt[VaryPar.nparm], "RADIUS= %f" );
			VaryPar.nvarxt[VaryPar.nparm] = 1;
		}

		/* log of radius is first number */
		VaryPar.vparm[0][VaryPar.nparm] = (float)log10(radius.Radius);
		++VaryPar.nparm;
	}


#	ifdef DEBUG_FUN
	fputs( " <->ParseRadius()\n", debug_fp );
#	endif
	return;
}

