/*prtmet print all line optical depths at end of iteration */
#include <string.h>
#include "cddefines.h"
#include "taulines.h"
#include "prttau.h"
#include "prtmet.h"
#include "elementnames.h"
#include "chlinelbl.h"
#include "printe82.h"
/*prme print heavy element line optical depths at end of calculation */
void prme(char *chDoIt, 
  float array[]);

void prtmet(void)
{
	long int i;

#	ifdef DEBUG_FUN
	fputs( "<+>prtmet()\n", debug_fp );
#	endif

	/* default is to not print optical depths, turn on with
	 * print optical depths on command */
	if( PrtTau.lgPrtTau )
	{

		/* 'In' means to zero counter, following elements mean nothing */
		prme("In",&TauLines[0][0]);

		/* print all lines lines */
		for( i=1; i <= NTAULINES; i++ )
		{
			prme(" c",&TauLines[i][0]);
		}

		for( i=0; i < nWindLine; i++ )
		{
			prme(" c",&TauLine2[i][0]);
		}

		/* this means print it */
		prme("  ",&TauLines[0][0]);
	}


#	ifdef DEBUG_FUN
	fputs( " <->prtmet()\n", debug_fp );
#	endif
	return;
}

/*prme print heavy element line optical depths at end of calculation */
#define	NLIM	6

/******************************************************************************
 *
 * prme - the helper routine for above 
 *
 ******************************************************************************/

void prme(char *chDoIt, 
  float array[])
{
	char chLabel[3];
	static char chLsav[NLIM][11];
	long int i;
	static long int n;
	static float tsav[NLIM] = {0.};

#	ifdef DEBUG_FUN
	fputs( "<+>prme()\n", debug_fp );
#	endif

	/* this routine is called by prtmet, and is used to print out
	 * line optical depths after model complete - it IS NOT a
	 * timing pace setter */

	/* label for line */
	strcpy( chLabel, elementnames.chElementSym[(long)(array[IPLNNELEM-1])-1] );

	if( strcmp(chDoIt,"In") == 0 )
	{
		n = 0;

	}
	else if( strcmp(chDoIt,"  ") == 0 )
	{
		/* NB number in following statement must agree with nlim */
		fprintf( ioQQQ, "   " );
		for( i=0; i < n; i++ )
		{
			fprintf( ioQQQ, "%10.10s",chLsav[i]);
			PrintE92(ioQQQ, tsav[i] );
			fprintf( ioQQQ, " ");
		}
		fprintf( ioQQQ, " \n" );

	}
	else if( array[IPLNTAUIN-1] > PrtTau.PrtTauFnt || array[IPLNTAUIN-1] < 0. )
	{
		/* array(ipLnTauIn) is the optical depth
		 * PrtTauFnt is threshold for printing it */
		if( n >= NLIM )
		{
			/* NB number in following statement must agree with nlim */
			fprintf( ioQQQ, "   " );
			for( i=0; i < NLIM; i++ )
			{
				fprintf( ioQQQ, "%10.10s",chLsav[i]);
				PrintE92(ioQQQ, tsav[i] );
				fprintf( ioQQQ, " ");
			}
			fprintf( ioQQQ, " \n" );
			n = 0;
		}

		strcpy( chLsav[n], chLineLbl(array) );
		tsav[n] = array[IPLNTAUIN-1];
		++n;
	}


#	ifdef DEBUG_FUN
	fputs( " <->prme()\n", debug_fp );
#	endif
	return;
}

